# Generated by Django 3.2.12 on 2022-03-24 19:24

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('cornerstone', '0016_auto_20220324_1550'),
    ]

    operations = [
        migrations.AlterField(
            model_name='cornerstonelearnerdatatransmissionaudit',
            name='course_completed',
            field=models.BooleanField(default=False),
        ),
    ]



# Generated by Django 3.2.20 on 2023-11-22 08:24

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('moodle', '0028_auto_20231116_1826'),
        ('moodle', '0029_auto_20231106_1233'),
    ]

    operations = [
    ]



from edx_rest_framework_extensions.auth.jwt.authentication import JwtAuthentication
from rest_framework import generics, permissions, viewsets
from rest_framework.authentication import SessionAuthentication
from rest_framework.renderers import JSONRenderer
from rest_framework.response import Response
from rest_framework.status import HTTP_200_OK

from django.utils.decorators import method_decorator

from enterprise.api.v1.decorators import require_at_least_one_query_parameter
from enterprise.utils import get_enterprise_customer_or_404
from integrated_channels.api.v1.blackboard.serializers import BlackboardConfigSerializer
from integrated_channels.api.v1.canvas.serializers import CanvasEnterpriseCustomerConfigurationSerializer
from integrated_channels.api.v1.cornerstone.serializers import CornerstoneConfigSerializer
from integrated_channels.api.v1.degreed2.serializers import Degreed2ConfigSerializer
from integrated_channels.api.v1.degreed.serializers import DegreedConfigSerializer
from integrated_channels.api.v1.mixins import PermissionRequiredForIntegratedChannelMixin
from integrated_channels.api.v1.moodle.serializers import MoodleConfigSerializer
from integrated_channels.api.v1.sap_success_factors.serializers import SAPSuccessFactorsConfigSerializer
from integrated_channels.blackboard.models import BlackboardEnterpriseCustomerConfiguration
from integrated_channels.canvas.models import CanvasEnterpriseCustomerConfiguration
from integrated_channels.cornerstone.models import CornerstoneEnterpriseCustomerConfiguration
from integrated_channels.degreed2.models import Degreed2EnterpriseCustomerConfiguration
from integrated_channels.degreed.models import DegreedEnterpriseCustomerConfiguration
from integrated_channels.integrated_channel.client import IntegratedChannelHealthStatus
from integrated_channels.integrated_channel.models import EnterpriseCustomerPluginConfiguration
from integrated_channels.moodle.models import MoodleEnterpriseCustomerConfiguration
from integrated_channels.sap_success_factors.models import SAPSuccessFactorsEnterpriseCustomerConfiguration
from integrated_channels.utils import get_enterprise_client_by_channel_code


class IntegratedChannelsBaseViewSet(viewsets.ViewSet):
    """
    API views for the ``integrated_channels`` base configurations endpoint.
    """
    permission_classes = (permissions.IsAuthenticated,)
    authentication_classes = (JwtAuthentication, SessionAuthentication,)

    permission_required = 'enterprise.can_access_admin_dashboard'

    @method_decorator(require_at_least_one_query_parameter('enterprise_customer'))
    # pylint: disable=unused-argument
    def list(self, request, *arg, **kwargs):
        """
        Enterprise Customer's Integrated Channels config list without pagination
        """
        enterprise_customer = get_enterprise_customer_or_404(self.request.query_params.get('enterprise_customer'))
        blackboard_configs = BlackboardEnterpriseCustomerConfiguration.objects.filter(
            enterprise_customer=enterprise_customer
        )
        blackboard_serializer = BlackboardConfigSerializer(blackboard_configs, many=True)

        canvas_configs = CanvasEnterpriseCustomerConfiguration.objects.filter(
            enterprise_customer=enterprise_customer
        )
        canvas_serializer = CanvasEnterpriseCustomerConfigurationSerializer(canvas_configs, many=True)

        cornerstone_configs = CornerstoneEnterpriseCustomerConfiguration.objects.filter(
            enterprise_customer=enterprise_customer
        )
        cornerstone_serializer = CornerstoneConfigSerializer(cornerstone_configs, many=True)

        degreed_configs = DegreedEnterpriseCustomerConfiguration.objects.filter(
            enterprise_customer=enterprise_customer
        )
        degreed_serializer = DegreedConfigSerializer(degreed_configs, many=True)

        degreed2_configs = Degreed2EnterpriseCustomerConfiguration.objects.filter(
            enterprise_customer=enterprise_customer
        )
        degreed2_serializer = Degreed2ConfigSerializer(degreed2_configs, many=True)

        moodle_configs = MoodleEnterpriseCustomerConfiguration.objects.filter(
            enterprise_customer=enterprise_customer
        )
        moodle_serializer = MoodleConfigSerializer(moodle_configs, many=True)

        sap_configs = SAPSuccessFactorsEnterpriseCustomerConfiguration.objects.filter(
            enterprise_customer=enterprise_customer
        )
        sap_serializer = SAPSuccessFactorsConfigSerializer(sap_configs, many=True)

        response = blackboard_serializer.data + canvas_serializer.data + cornerstone_serializer.data + \
            degreed_serializer.data + degreed2_serializer.data + moodle_serializer.data + sap_serializer.data
        return Response(response)


class IntegratedChannelHealthCheckView(generics.ListAPIView):
    """
        **Use Cases**

            Perform Health Check on user's connection

        **Example Requests**

            GET /config/health-check

        **Query Parameters for GET**

            * channel_code: The channel code of the CanvasEnterpriseCustomerConfiguration
            * uuid: The uuid of the CanvasEnterpriseCustomerConfiguration

        **Supported Channels**
        canvas

        **Response Values**

            * is_healthy: Whether the Canvas connection is healthy
            * health_status: Status code for health status


    """
    renderer_classes = [JSONRenderer, ]

    @method_decorator(require_at_least_one_query_parameter('channel_code'))
    @method_decorator(require_at_least_one_query_parameter('uuid'))
    def get(self, request, *args, **kwargs):
        """
        Enterprise Customer's Integrated Channel health check
        """
        channel_code = request.query_params.get('channel_code').lower()
        uuid = request.query_params.get('uuid')

        # Get enterprise configuration
        config_type = EnterpriseCustomerPluginConfiguration.get_class_by_channel_code(channel_code)
        config = config_type.objects.get(uuid=uuid)
        client_type = get_enterprise_client_by_channel_code(channel_code)
        client = client_type(config)

        health_status = client.health_check()
        is_healthy = health_status == IntegratedChannelHealthStatus.HEALTHY
        payload = {'is_healthy': is_healthy, 'health_status': health_status.value}
        return Response(payload, status=HTTP_200_OK)



# Generated by Django 2.2.13 on 2020-06-30 20:22

from django.db import migrations


# this field is new, there is no data in prod that we have to update
def move_data_from_marked_done_to_saved_for_later(apps, schema_editor):
   pass


def move_data_from_saved_for_later_to_marked_done(apps, schema_editor):
    pass

class Migration(migrations.Migration):

    dependencies = [
        ('enterprise', '0100_add_licensed_enterprise_course_enrollment'),
    ]

    operations = [
        migrations.RunPython(move_data_from_marked_done_to_saved_for_later, move_data_from_saved_for_later_to_marked_done)
    ]



# Generated by Django 3.2.15 on 2022-09-13 20:18

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('xapi', '0007_auto_20220405_2311'),
    ]

    operations = [
        migrations.AddField(
            model_name='xapilearnerdatatransmissionaudit',
            name='friendly_status_message',
            field=models.CharField(blank=True, default=None, help_text='A user-friendly API response status message.', max_length=255, null=True),
        ),
    ]



# Generated by Django 3.2.23 on 2023-11-30 22:39

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('enterprise', '0196_backfill_sso_marked_authorized'),
    ]

    operations = [
        migrations.AlterField(
            model_name='enterprisecustomerssoconfiguration',
            name='marked_authorized',
            field=models.BooleanField(default=False, help_text='Whether admin has indicated the service provider metadata was uploaded.'),
        ),
        migrations.AlterField(
            model_name='historicalenterprisecustomerssoconfiguration',
            name='marked_authorized',
            field=models.BooleanField(default=False, help_text='Whether admin has indicated the service provider metadata was uploaded.'),
        ),
    ]



# Generated by Django 3.2.23 on 2024-02-27 22:27

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('enterprise', '0200_enterprisegroup_enterprisegroupmembership_historicalenterprisegroup_historicalenterprisegroupmembers'),
    ]

    operations = [
        migrations.AddField(
            model_name='enterprisegroup',
            name='is_removed',
            field=models.BooleanField(default=False),
        ),
        migrations.AddField(
            model_name='enterprisegroupmembership',
            name='is_removed',
            field=models.BooleanField(default=False),
        ),
        migrations.AddField(
            model_name='historicalenterprisegroup',
            name='is_removed',
            field=models.BooleanField(default=False),
        ),
        migrations.AddField(
            model_name='historicalenterprisegroupmembership',
            name='is_removed',
            field=models.BooleanField(default=False),
        ),
    ]



# Generated by Django 2.2.17 on 2020-12-15 08:58

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('enterprise', '0116_auto_20201208_1759'),
    ]

    operations = [
        migrations.AlterField(
            model_name='enterprisecustomeridentityprovider',
            name='enterprise_customer',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='enterprise_customer_identity_providers', to='enterprise.EnterpriseCustomer'),
        ),
    ]



"""
Assist integrated channels with retrieving content metadata.

Module contains resources for integrated channels to retrieve all the
metadata for content contained in the catalogs associated with a particular
enterprise customer.
"""

import sys
from logging import getLogger

from django.apps import apps
from django.conf import settings
from django.db.models import Q
from django.utils import timezone

from enterprise.api_client.enterprise_catalog import EnterpriseCatalogApiClient
from enterprise.constants import (
    EXEC_ED_CONTENT_DESCRIPTION_TAG,
    EXEC_ED_COURSE_TYPE,
    IC_CREATE_ACTION,
    IC_UPDATE_ACTION,
    TRANSMISSION_MARK_CREATE,
)
from enterprise.utils import get_content_metadata_item_id
from integrated_channels.integrated_channel.exporters import Exporter
from integrated_channels.utils import generate_formatted_log, truncate_item_dicts

LOGGER = getLogger(__name__)


class ContentMetadataExporter(Exporter):
    """
    Base class for content metadata exporters.
    """

    # DATA_TRANSFORM_MAPPING is used to map the content metadata field names expected by the integrated channel
    # to the edX content metadata schema. The values contained in the dict will be used as keys to access values
    # in each content metadata item dict which is being exported.
    #
    # Example:
    #     {
    #         'contentID': 'key',
    #         'courseTitle': 'title'
    #     }
    #
    #     Defines a transformation of the content metadata item to:
    #
    #     {
    #         'contentID': content_metadata_item['key'],
    #         'courseTitle': content_metadata_item['title']
    #     }
    #
    # Subclasses should override this class variable. By default, the edX content metadata schema is returned in
    # its entirety.
    #
    # In addition, subclasses can implement transform functions which receive a content metadata item for more
    # complicated field transformations. These functions can be content type-specific or generic for all content
    # types.
    #
    # Example:
    #     DATA_TRANSFORM_MAPPING = {
    #         'coursePrice': 'price'
    #     }
    #     # Content type-specific transformer
    #     def transform_course_price(self, course):
    #         return course['course_runs'][0]['seats']['verified]['price']
    #     # Generic transformer
    #     def transform_provider_id(self, course):
    #         return self.enterprise_configuration.provider_id
    #
    # TODO: Move this to the EnterpriseCustomerPluginConfiguration model as a JSONField.
    DATA_TRANSFORM_MAPPING = {}
    SKIP_KEY_IF_NONE = False
    LAST_24_HRS = timezone.now() - timezone.timedelta(hours=24)

    def __init__(self, user, enterprise_configuration):
        """
        Initialize the exporter.
        """
        super().__init__(user, enterprise_configuration)
        self.enterprise_catalog_api = EnterpriseCatalogApiClient(self.user)

    def _log_info(self, msg, course_or_course_run_key=None):
        LOGGER.info(
            generate_formatted_log(
                channel_name=self.enterprise_configuration.channel_code(),
                enterprise_customer_uuid=self.enterprise_configuration.enterprise_customer.uuid,
                course_or_course_run_key=course_or_course_run_key,
                plugin_configuration_id=self.enterprise_configuration.id,
                message=msg
            )
        )

    def _log_exception(self, msg, course_or_course_run_key=None):
        LOGGER.exception(
            generate_formatted_log(
                channel_name=self.enterprise_configuration.channel_code(),
                enterprise_customer_uuid=self.enterprise_configuration.enterprise_customer.uuid,
                course_or_course_run_key=course_or_course_run_key,
                plugin_configuration_id=self.enterprise_configuration.id,
                message=msg
            )
        )

    def _get_catalog_content_keys(self, enterprise_customer_catalog=None):
        """
        Retrieve all non-deleted content transmissions under a given customer's catalog
        """
        ContentMetadataItemTransmission = apps.get_model(
            'integrated_channel',
            'ContentMetadataItemTransmission'
        )

        # created, not deleted, no error code
        base_content_query = Q(
            enterprise_customer=self.enterprise_configuration.enterprise_customer,
            integrated_channel_code=self.enterprise_configuration.channel_code(),
            plugin_configuration_id=self.enterprise_configuration.id,
            remote_deleted_at__isnull=True,
            remote_created_at__isnull=False,
        )
        # enterprise_customer_catalog filter is optional
        if enterprise_customer_catalog is not None:
            base_content_query.add(Q(enterprise_customer_catalog_uuid=enterprise_customer_catalog.uuid), Q.AND)
        # api_response_status_code can be null, treat that as successful, otherwise look for less than http 400
        base_content_query.add(Q(api_response_status_code__isnull=True) | Q(api_response_status_code__lt=400), Q.AND)

        # query for records that have a created at date and a failure status code
        failed_query = Q(
            enterprise_customer=self.enterprise_configuration.enterprise_customer,
            integrated_channel_code=self.enterprise_configuration.channel_code(),
            plugin_configuration_id=self.enterprise_configuration.id,
            api_response_status_code__gte=400,
            remote_created_at__isnull=False,
        )
        # filter only records who have failed to delete or update, meaning we know they exist on the customer's
        # instance and require some kind of action
        failed_query.add(Q(remote_deleted_at__isnull=False) | Q(remote_updated_at__isnull=False), Q.AND)
        # enterprise_customer_catalog filter is optional
        if enterprise_customer_catalog is not None:
            failed_query.add(Q(enterprise_customer_catalog_uuid=enterprise_customer_catalog.uuid), Q.AND)

        # base query OR failed delete query
        final_content_query = Q(base_content_query | failed_query)

        past_transmissions = ContentMetadataItemTransmission.objects.filter(
            final_content_query
        ).values('content_id')
        if not past_transmissions:
            return []
        return [key.get('content_id') for key in past_transmissions]

    def _check_matched_content_updated_at(
        self,
        enterprise_customer_catalog,
        matched_items,
        force_retrieve_all_catalogs
    ):
        """
        Take a list of content keys and their respective last updated time and build a mapping between content keys and
        past content metadata transmission record when the last updated time comes after the last updated time of the
        record.

        Args:
            enterprise_customer_catalog (EnterpriseCustomerCatalog): The enterprise catalog object

            matched_items (list): A list of dicts containing content keys and the last datetime that the respective
                content was updated

            force_retrieve_all_catalogs (Bool): If set to True, all content under the catalog will be retrieved,
                regardless of the last updated at time
        """
        ContentMetadataItemTransmission = apps.get_model(
            'integrated_channel',
            'ContentMetadataItemTransmission'
        )
        items_to_update = {}
        for matched_item in matched_items:
            content_id = matched_item.get('content_key')
            content_last_changed = matched_item.get('date_updated')
            incomplete_transmission = ContentMetadataItemTransmission.incomplete_update_transmissions(
                enterprise_customer=self.enterprise_configuration.enterprise_customer,
                plugin_configuration_id=self.enterprise_configuration.id,
                integrated_channel_code=self.enterprise_configuration.channel_code(),
                content_id=content_id,
            ).first()
            if incomplete_transmission:
                self._log_info(
                    'Found an unsent content update record while creating record. '
                    'Including record.',
                    course_or_course_run_key=content_id
                )
                incomplete_transmission.mark_for_update()
                items_to_update[content_id] = incomplete_transmission
            else:
                content_query = Q(
                    enterprise_customer=self.enterprise_configuration.enterprise_customer,
                    integrated_channel_code=self.enterprise_configuration.channel_code(),
                    enterprise_customer_catalog_uuid=enterprise_customer_catalog.uuid,
                    plugin_configuration_id=self.enterprise_configuration.id,
                    content_id=content_id,
                    remote_deleted_at__isnull=True,
                    remote_created_at__isnull=False,
                )
                content_query.add(Q(remote_errored_at__lt=self.LAST_24_HRS) | Q(
                    remote_errored_at__isnull=True), Q.AND)
                # If not force_retrieve_all_catalogs, filter content records where `content last changed` is less than
                # the matched item's `date_updated`, otherwise select the row regardless of what the updated at time is.
                if not force_retrieve_all_catalogs:
                    last_changed_query = Q(content_last_changed__lt=content_last_changed)
                    last_changed_query.add(Q(content_last_changed__isnull=True), Q.OR)
                    get_marked_for = Q(marked_for='update')
                    get_marked_for.add(last_changed_query, Q.OR)
                    content_query.add(get_marked_for, Q.AND)
                items_to_update_query = ContentMetadataItemTransmission.objects.filter(content_query)
                item = items_to_update_query.first()
                if item:
                    item.mark_for_update()
                    items_to_update[content_id] = item
        return items_to_update

    def _check_matched_content_to_create(
        self,
        enterprise_customer_catalog,
        matched_items
    ):
        """
        Take a list of content keys and create ContentMetadataItemTransmission records. When existed soft-deleted
        records exist, resurrect them. When created but not-yet-transmitted records exist, include them.

        Args:
            enterprise_customer_catalog (EnterpriseCustomerCatalog): The enterprise catalog object

            matched_items (list): A list of dicts containing content keys and the last datetime that the respective
                content was updated

            force_retrieve_all_catalogs (Bool): If set to True, all content under the catalog will be retrieved,
                regardless of the last updated at time
        """
        ContentMetadataItemTransmission = apps.get_model(
            'integrated_channel',
            'ContentMetadataItemTransmission'
        )
        items_to_create = {}
        for matched_item in matched_items:
            content_id = matched_item.get('content_key')
            content_last_changed = matched_item.get('date_updated')
            past_deleted_transmission = ContentMetadataItemTransmission.deleted_transmissions(
                enterprise_customer=self.enterprise_configuration.enterprise_customer,
                plugin_configuration_id=self.enterprise_configuration.id,
                integrated_channel_code=self.enterprise_configuration.channel_code(),
                content_id=content_id,
            ).first()
            if past_deleted_transmission:
                self._log_info(
                    'Found previously deleted content record while creating record. '
                    'Marking record as active.',
                    course_or_course_run_key=content_id
                )
                past_deleted_transmission.prepare_to_recreate(content_last_changed, enterprise_customer_catalog.uuid)
                items_to_create[content_id] = past_deleted_transmission
            else:
                incomplete_transmission = ContentMetadataItemTransmission.incomplete_create_transmissions(
                    enterprise_customer=self.enterprise_configuration.enterprise_customer,
                    plugin_configuration_id=self.enterprise_configuration.id,
                    integrated_channel_code=self.enterprise_configuration.channel_code(),
                    content_id=content_id,
                ).first()
                if incomplete_transmission:
                    incomplete_transmission.mark_for_create()
                    items_to_create[content_id] = incomplete_transmission
                else:
                    new_transmission = ContentMetadataItemTransmission(
                        enterprise_customer=self.enterprise_configuration.enterprise_customer,
                        integrated_channel_code=self.enterprise_configuration.channel_code(),
                        content_id=content_id,
                        channel_metadata=None,
                        content_last_changed=content_last_changed,
                        enterprise_customer_catalog_uuid=enterprise_customer_catalog.uuid,
                        plugin_configuration_id=self.enterprise_configuration.id,
                        marked_for=TRANSMISSION_MARK_CREATE
                    )
                    new_transmission.save()
                    items_to_create[content_id] = new_transmission
        return items_to_create

    def _get_catalog_diff(
        self,
        enterprise_catalog,
        content_keys,
        force_retrieve_all_catalogs,
        max_item_count
    ):
        """
        From the enterprise catalog API, request a catalog diff based off of a list of content keys. Using the diff,
        retrieve past content metadata transmission records for update and delete payloads.
        """
        items_to_create, items_to_delete, matched_items = self.enterprise_catalog_api.get_catalog_diff(
            enterprise_catalog,
            content_keys
        )

        # Fetch all existing, non-deleted transmission audit content keys for the customer/configuration
        existing_content_keys = set(self._get_catalog_content_keys())
        unique_new_items_to_create = []

        # We need to remove any potential create transmissions if the content already exists on the customer's instance
        # under a different catalog
        for item in items_to_create:
            # If the catalog system has indicated that the content is new and needs creating, we need to check if the
            # content already exists on the customer's instance under a different catalog. If it does, we need to
            # check if the content key exists as an orphaned transmission record for this customer and config,
            # indicating that the content was previously created but then the config under which it was created was
            # deleted.
            content_key = item.get('content_key')
            orphaned_content = self._get_customer_config_orphaned_content(
                max_set_count=1,
                content_key=content_key
            ).first()

            # if it does exist as an orphaned content record: 1) don't add the item to the list of items to create,
            # 2) swap the catalog uuid of the transmission audit associated with the orphaned record, and 3) mark the
            # orphaned record resolved
            if orphaned_content:
                ContentMetadataItemTransmission = apps.get_model(
                    'integrated_channel',
                    'ContentMetadataItemTransmission',
                )
                ContentMetadataItemTransmission.objects.filter(
                    integrated_channel_code=self.enterprise_configuration.channel_code(),
                    plugin_configuration_id=self.enterprise_configuration.id,
                    content_id=content_key
                ).update(
                    enterprise_customer_catalog_uuid=enterprise_catalog.uuid
                )

                self._log_info(
                    'Found an orphaned content record while creating. '
                    'Swapping catalog uuid and marking record as resolved.',
                    course_or_course_run_key=content_key
                )
                orphaned_content.resolved = True
                orphaned_content.save()

            # if the item to create doesn't exist as an orphaned piece of content, do all the normal checks
            elif content_key not in existing_content_keys:
                unique_new_items_to_create.append(item)

        content_to_create = self._check_matched_content_to_create(
            enterprise_catalog,
            unique_new_items_to_create
        )
        content_to_update = self._check_matched_content_updated_at(
            enterprise_catalog,
            matched_items,
            force_retrieve_all_catalogs
        )
        content_to_delete = self._check_matched_content_to_delete(
            enterprise_catalog,
            items_to_delete
        )

        truncated_create, truncated_update, truncated_delete = truncate_item_dicts(
            content_to_create,
            content_to_update,
            content_to_delete,
            max_item_count
        )

        return truncated_create, truncated_update, truncated_delete

    def _check_matched_content_to_delete(self, enterprise_customer_catalog, items):
        """
        Retrieve all past content metadata transmission records that have a `content_id` contained within a provided
        list.
        renamed from _retrieve_past_transmission_content
        """
        ContentMetadataItemTransmission = apps.get_model(
            'integrated_channel',
            'ContentMetadataItemTransmission'
        )

        items_to_delete = {}
        for item in items:
            content_id = item.get('content_key')

            incomplete_transmission = ContentMetadataItemTransmission.incomplete_delete_transmissions(
                enterprise_customer=self.enterprise_configuration.enterprise_customer,
                plugin_configuration_id=self.enterprise_configuration.id,
                integrated_channel_code=self.enterprise_configuration.channel_code(),
                content_id=content_id,
            ).first()

            if incomplete_transmission:
                self._log_info(
                    'Found an unsent content delete record while deleting record. '
                    'Including record.',
                    course_or_course_run_key=content_id
                )
                incomplete_transmission.mark_for_delete()
                items_to_delete[content_id] = incomplete_transmission
            else:
                past_content_query = Q(
                    enterprise_customer=self.enterprise_configuration.enterprise_customer,
                    integrated_channel_code=self.enterprise_configuration.channel_code(),
                    enterprise_customer_catalog_uuid=enterprise_customer_catalog.uuid,
                    plugin_configuration_id=self.enterprise_configuration.id,
                    content_id=content_id
                )
                past_content_query.add(Q(remote_errored_at__lt=self.LAST_24_HRS) | Q(
                    remote_errored_at__isnull=True), Q.AND)
                past_content = ContentMetadataItemTransmission.objects.filter(
                    past_content_query).first()
                if past_content:
                    past_content.mark_for_delete()
                    items_to_delete[content_id] = past_content
                else:
                    self._log_info(
                        'Could not find a content record while deleting record. '
                        'Skipping record.',
                        course_or_course_run_key=content_id
                    )
        return items_to_delete

    def _get_customer_config_orphaned_content(self, max_set_count, content_key=None):
        """
        Helper method to retrieve the customer's orphaned content metadata items.
        """
        OrphanedContentTransmissions = apps.get_model(
            'integrated_channel',
            'OrphanedContentTransmissions'
        )
        content_query = Q(content_id=content_key) if content_key else Q()
        base_query = Q(
            integrated_channel_code=self.enterprise_configuration.channel_code(),
            plugin_configuration_id=self.enterprise_configuration.id,
            resolved=False,
        ) & content_query

        # Grab orphaned content metadata items for the customer, ordered by oldest to newest
        orphaned_content = OrphanedContentTransmissions.objects.filter(base_query)
        ordered_and_chunked_orphaned_content = orphaned_content.order_by('created')[:max_set_count]
        return ordered_and_chunked_orphaned_content

    def _sanitize_and_set_item_metadata(self, item, metadata, action):
        """
        Helper method to sanitize and set the metadata of an audit record according to
        the provided action being performed on the item.
        """
        metadata_transformed_for_exec_ed = self._transform_exec_ed_content(metadata)
        transformed_item = self._transform_item(metadata_transformed_for_exec_ed, action=action)

        item.channel_metadata = transformed_item
        item.content_title = metadata.get('title')
        item.content_last_changed = metadata.get('content_last_modified')
        item.save()

    def export(self, **kwargs):
        """
        Export transformed content metadata if there has been an update to the consumer's catalogs
        """
        enterprise_customer_catalogs = self.enterprise_configuration.customer_catalogs_to_transmit or \
            self.enterprise_customer.enterprise_customer_catalogs.all()

        # a maximum number of changes/payloads to export at once
        # default to something huge to simplify logic, the max system int size
        max_payload_count = kwargs.get('max_payload_count', sys.maxsize)

        self._log_info(
            f'Beginning export for customer: {self.enterprise_customer.uuid}, with '
            f'max_payload_count of {max_payload_count}, and catalogs: '
            f'{enterprise_customer_catalogs}'
        )

        create_payload = {}
        update_payload = {}
        delete_payload = {}
        for enterprise_customer_catalog in enterprise_customer_catalogs:

            # if we're already at the max in a multi-catalog situation, break out
            if len(create_payload) + len(update_payload) + len(delete_payload) >= max_payload_count:
                self._log_info(f'Reached max_payload_count of {max_payload_count} breaking.')
                break

            content_keys = self._get_catalog_content_keys(enterprise_customer_catalog)

            self._log_info(
                f'Retrieved {len(content_keys)} content keys for past transmissions to customer: '
                f'{self.enterprise_customer.uuid} under catalog: {enterprise_customer_catalog.uuid}.'
            )

            # From the saved content records, use the enterprise catalog API to determine what needs sending
            items_to_create, items_to_update, items_to_delete = self._get_catalog_diff(
                enterprise_customer_catalog,
                content_keys,
                kwargs.get('force_retrieve_all_catalogs', False),
                max_payload_count
            )

            content_keys_filter = list(items_to_create.keys()) + list(items_to_update.keys())
            if content_keys_filter:
                content_metadata_items = self.enterprise_catalog_api.get_content_metadata(
                    self.enterprise_customer,
                    [enterprise_customer_catalog],
                    content_keys_filter,
                )
                key_to_content_metadata_mapping = {
                    get_content_metadata_item_id(item): item for item in content_metadata_items
                }
            for key, item in items_to_create.items():
                try:
                    self._sanitize_and_set_item_metadata(item, key_to_content_metadata_mapping[key], IC_CREATE_ACTION)
                except Exception as exc:
                    self._log_exception(
                        f'Failed to sanitize and set item metadata for item: {item}, with content key: '
                        f'{key}, and content metadata: {key_to_content_metadata_mapping[key]}, action: '
                        f'{IC_CREATE_ACTION}. Exception: {exc}'
                    )
                    raise exc

                # Sanity check
                item.enterprise_customer_catalog_uuid = enterprise_customer_catalog.uuid
                item.save()

                create_payload[key] = item
            for key, item in items_to_update.items():
                try:
                    self._sanitize_and_set_item_metadata(item, key_to_content_metadata_mapping[key], IC_UPDATE_ACTION)
                except Exception as exc:
                    self._log_exception(
                        f'Failed to sanitize and set item metadata for item: {item}, with content key: '
                        f'{key}, and content metadata: {key_to_content_metadata_mapping[key]}, action: '
                        f'{IC_UPDATE_ACTION}. Exception: {exc}'
                    )
                    raise exc

                # Sanity check
                item.enterprise_customer_catalog_uuid = enterprise_customer_catalog.uuid
                item.save()

                update_payload[key] = item
            for key, item in items_to_delete.items():
                metadata = self._apply_delete_transformation(item.channel_metadata)
                item.channel_metadata = metadata
                item.save()
                delete_payload[key] = item

        # If we're not at the max payload count, we can check for orphaned content and shove it in the delete payload
        current_payload_count = len(create_payload) + len(update_payload) + len(delete_payload)

        self._log_info(
            f'Exporter finished iterating over catalogs for customer: {self.enterprise_customer.uuid},'
            f'with a current payload count of: {current_payload_count}. Is there room for orphaned content'
            f'in the exporter payload?: {current_payload_count < max_payload_count}'
        )

        if current_payload_count < max_payload_count:
            self._log_info(
                f'Exporter has {max_payload_count - current_payload_count} slots left in the payload for customer: '
                f'{self.enterprise_customer.uuid}, searching for orphaned content to append'
            )

            space_left_in_payload = max_payload_count - current_payload_count
            orphaned_content_to_delete = self._get_customer_config_orphaned_content(
                max_set_count=space_left_in_payload,
            )

            for orphaned_item in orphaned_content_to_delete:
                # log the content that would have been deleted because it's orphaned
                self._log_info(
                    f'Exporter intends to delete orphaned content for customer: {self.enterprise_customer.uuid}, '
                    f'config {self.enterprise_configuration.channel_code}-{self.enterprise_configuration} with '
                    f'content_id: {orphaned_item.content_id}'
                )
                if getattr(settings, "ALLOW_ORPHANED_CONTENT_REMOVAL", False):
                    delete_payload[orphaned_item.content_id] = orphaned_item.transmission

        self._log_info(
            f'Exporter finished for customer: {self.enterprise_customer.uuid} with payloads- create_payload: '
            f'{create_payload}, update_payload: {update_payload}, delete_payload: {delete_payload}'
        )

        # collections of ContentMetadataItemTransmission objects
        return create_payload, update_payload, delete_payload

    def _apply_delete_transformation(self, metadata):
        """
        Base implementation of a delete transformation method. This method is designed to be a NOOP as it is up to the
        individual channel to define specific transformations for the channel metadata payload of transmissions
        intended for deletion.
        """
        return metadata

    def _transform_exec_ed_content(self, content):
        """
        Transform only executive education course type content to add executive education identifying tags to both the
        title and description of the content
        """
        if content.get('course_type') == EXEC_ED_COURSE_TYPE:
            if title := content.get('title'):
                content['title'] = "ExecEd: " + title
            if description := content.get('full_description'):
                content['full_description'] = EXEC_ED_CONTENT_DESCRIPTION_TAG + description
        return content

    def _transform_item(self, content_metadata_item, action):
        """
        Transform the provided content metadata item to the schema expected by the integrated channel.
        """
        content_metadata_type = content_metadata_item['content_type']
        transformed_item = {}
        for integrated_channel_schema_key, edx_data_schema_key in self.DATA_TRANSFORM_MAPPING.items():
            # Look for transformer functions defined on subclasses.
            # Favor content type-specific functions.
            transformer = (
                getattr(self, f'transform_{content_metadata_type}_{edx_data_schema_key}', None)
                or
                getattr(self, f'transform_{edx_data_schema_key}', None)
            )

            transformer_for_action = (
                getattr(self, f'transform_for_action_{content_metadata_type}_{edx_data_schema_key}', None)
                or
                getattr(self, f'transform_for_action_{edx_data_schema_key}', None)
            )

            if transformer:
                transformed_value = transformer(content_metadata_item)
            elif transformer_for_action:
                transformed_value = transformer_for_action(content_metadata_item, action)
            else:
                # The concrete subclass does not define an override for the given field,
                # so just use the data key to index the content metadata item dictionary.
                try:
                    transformed_value = content_metadata_item[edx_data_schema_key]
                except KeyError:
                    # There may be a problem with the DATA_TRANSFORM_MAPPING on
                    # the concrete subclass or the concrete subclass does not implement
                    # the appropriate field transformer function.
                    self._log_exception(
                        f'Failed to transform content metadata item field {edx_data_schema_key} '
                        f'for {self.enterprise_customer.name}: {content_metadata_item}'
                    )
                    continue

            if transformed_value is None and self.SKIP_KEY_IF_NONE:
                continue
            transformed_item[integrated_channel_schema_key] = transformed_value

        return transformed_item

    def update_content_transmissions_catalog_uuids(self):
        """
        Retrieve all content under the enterprise customer's catalog(s) and update all past transmission audits to have
        it's associated catalog uuid.
        """
        enterprise_customer_catalogs = self.enterprise_configuration.customer_catalogs_to_transmit or \
            self.enterprise_customer.enterprise_customer_catalogs.all()

        for enterprise_customer_catalog in enterprise_customer_catalogs:
            content_metadata_items = self.enterprise_catalog_api.get_content_metadata(
                self.enterprise_customer,
                [enterprise_customer_catalog]
            )
            content_ids = [get_content_metadata_item_id(item) for item in content_metadata_items]
            ContentMetadataItemTransmission = apps.get_model(
                'integrated_channel',
                'ContentMetadataItemTransmission'
            )
            transmission_items = ContentMetadataItemTransmission.objects.filter(
                enterprise_customer=self.enterprise_configuration.enterprise_customer,
                integrated_channel_code=self.enterprise_configuration.channel_code(),
                plugin_configuration_id=self.enterprise_configuration.id,
                content_id__in=content_ids
            )
            self._log_info(
                f'Found {len(transmission_items)} past content transmissions that need to be updated with their '
                f'respective catalog (catalog: {enterprise_customer_catalog.uuid}) UUIDs'
            )
            for item in transmission_items:
                item.enterprise_customer_catalog_uuid = enterprise_customer_catalog.uuid
                item.save()



# Generated by Django 3.2.20 on 2023-09-28 15:30

from django.db import migrations
import fernet_fields.fields


class Migration(migrations.Migration):

    dependencies = [
        ('moodle', '0027_alter_historicalmoodleenterprisecustomerconfiguration_options'),
    ]

    operations = [
        migrations.AddField(
            model_name='historicalmoodleenterprisecustomerconfiguration',
            name='decrypted_password',
            field=fernet_fields.fields.EncryptedCharField(blank=True, help_text="The encrypted API user's password used to obtain new tokens. It will be encrypted when stored in the database.", max_length=255, null=True, verbose_name='Encrypted Webservice Password'),
        ),
        migrations.AddField(
            model_name='historicalmoodleenterprisecustomerconfiguration',
            name='decrypted_token',
            field=fernet_fields.fields.EncryptedCharField(blank=True, help_text="The encrypted API user's token used to obtain new tokens. It will be encrypted when stored in the database.", max_length=255, null=True, verbose_name='Encrypted Webservice Token'),
        ),
        migrations.AddField(
            model_name='historicalmoodleenterprisecustomerconfiguration',
            name='decrypted_username',
            field=fernet_fields.fields.EncryptedCharField(blank=True, help_text="The encrypted API user's username used to obtain new tokens. It will be encrypted when stored in the database.", max_length=255, null=True, verbose_name='Encrypted Webservice Username'),
        ),
        migrations.AddField(
            model_name='moodleenterprisecustomerconfiguration',
            name='decrypted_password',
            field=fernet_fields.fields.EncryptedCharField(blank=True, help_text="The encrypted API user's password used to obtain new tokens. It will be encrypted when stored in the database.", max_length=255, null=True, verbose_name='Encrypted Webservice Password'),
        ),
        migrations.AddField(
            model_name='moodleenterprisecustomerconfiguration',
            name='decrypted_token',
            field=fernet_fields.fields.EncryptedCharField(blank=True, help_text="The encrypted API user's token used to obtain new tokens. It will be encrypted when stored in the database.", max_length=255, null=True, verbose_name='Encrypted Webservice Token'),
        ),
        migrations.AddField(
            model_name='moodleenterprisecustomerconfiguration',
            name='decrypted_username',
            field=fernet_fields.fields.EncryptedCharField(blank=True, help_text="The encrypted API user's username used to obtain new tokens. It will be encrypted when stored in the database.", max_length=255, null=True, verbose_name='Encrypted Webservice Username'),
        ),
    ]



"""
Database models for Enterprise Integrated Channel Canvas.
"""

import json
import uuid
from logging import getLogger

from six.moves.urllib.parse import urljoin

from django.conf import settings
from django.db import models
from django.utils.translation import gettext_lazy as _

from integrated_channels.canvas.exporters.content_metadata import CanvasContentMetadataExporter
from integrated_channels.canvas.exporters.learner_data import CanvasLearnerExporter
from integrated_channels.canvas.transmitters.content_metadata import CanvasContentMetadataTransmitter
from integrated_channels.canvas.transmitters.learner_data import CanvasLearnerTransmitter
from integrated_channels.integrated_channel.models import (
    EnterpriseCustomerPluginConfiguration,
    LearnerDataTransmissionAudit,
)
from integrated_channels.utils import is_valid_url

LOGGER = getLogger(__name__)
LMS_OAUTH_REDIRECT_URL = urljoin(settings.LMS_ROOT_URL, '/canvas/oauth-complete')


class CanvasEnterpriseCustomerConfiguration(EnterpriseCustomerPluginConfiguration):
    """
    The Enterprise-specific configuration we need for integrating with Canvas.

    Based on: https://canvas.instructure.com/doc/api/file.oauth.html#oauth2-flow-3
    """

    client_id = models.CharField(
        max_length=255,
        blank=True,
        default='',
        verbose_name="API Client ID",
        help_text=_(
            "The API Client ID provided to edX by the enterprise customer to be used to make API "
            "calls to Canvas on behalf of the customer."
        )
    )

    client_secret = models.CharField(
        max_length=255,
        blank=True,
        default='',
        verbose_name="API Client Secret",
        help_text=_(
            "The API Client Secret provided to edX by the enterprise customer to be used to make "
            " API calls to Canvas on behalf of the customer."
        )
    )

    canvas_account_id = models.BigIntegerField(
        null=True,
        blank=True,
        verbose_name="Canvas Account Number",
        help_text=_("Account number to use during api calls. Called account_id in canvas. "
                    " Required to create courses etc.")
    )

    canvas_base_url = models.CharField(
        max_length=255,
        blank=True,
        default='',
        verbose_name="Canvas Base URL",
        help_text=_("The base URL used for API requests to Canvas, i.e. https://instructure.com.")
    )

    refresh_token = models.CharField(
        max_length=255,
        blank=True,
        verbose_name="Oauth2 Refresh Token",
        help_text=_("The refresh token provided by Canvas along with the access token request, used to "
                    "re-request the access tokens over multiple client sessions.")
    )

    # overriding base model field, to use chunk size 1 default
    transmission_chunk_size = models.IntegerField(
        default=1,
        help_text=_("The maximum number of data items to transmit to the integrated channel "
                    "with each request.")
    )

    uuid = models.UUIDField(
        unique=True,
        default=uuid.uuid4,
        editable=False,
        help_text=(
            "A UUID for use in public-facing urls such as oauth state variables."
        )
    )

    class Meta:
        app_label = 'canvas'

    @property
    def oauth_authorization_url(self):
        """
        Returns: the oauth authorization url when the canvas_base_url and client_id are available.

        Args:
            obj: The instance of CanvasEnterpriseCustomerConfiguration
                being rendered with this admin form.
        """
        if self.canvas_base_url and self.client_id:
            return (f'{self.canvas_base_url}/login/oauth2/auth'
                    f'?redirect_uri={LMS_OAUTH_REDIRECT_URL}&'
                    f'response_type=code&'
                    f'client_id={self.client_id}&state={self.uuid}')
        else:
            return None

    @property
    def is_valid(self):
        """
        Returns whether or not the configuration is valid and ready to be activated

        Args:
            obj: The instance of CanvasEnterpriseCustomerConfiguration
                being rendered with this admin form.
        """
        missing_items = {'missing': []}
        incorrect_items = {'incorrect': []}
        if not self.client_id:
            missing_items.get('missing').append('client_id')
        if not self.client_secret:
            missing_items.get('missing').append('client_secret')
        if not self.canvas_base_url:
            missing_items.get('missing').append('canvas_base_url')
        if not self.canvas_account_id:
            missing_items.get('missing').append('canvas_account_id')
        if not self.refresh_token:
            missing_items.get('missing').append('refresh_token')
        if not is_valid_url(self.canvas_base_url):
            incorrect_items.get('incorrect').append('canvas_base_url')
        if len(self.display_name) > 30:
            incorrect_items.get('incorrect').append('display_name')
        return missing_items, incorrect_items

    def __str__(self):
        """
        Return human-readable string representation.
        """
        return "<CanvasEnterpriseCustomerConfiguration for Enterprise {enterprise_name}>".format(
            enterprise_name=self.enterprise_customer.name
        )

    def __repr__(self):
        """
        Return uniquely identifying string representation.
        """
        return self.__str__()

    @staticmethod
    def channel_code():
        """
        Returns an capitalized identifier for this channel class, unique among subclasses.
        """
        return 'CANVAS'

    def get_learner_data_transmitter(self):
        return CanvasLearnerTransmitter(self)

    def get_learner_data_exporter(self, user):
        return CanvasLearnerExporter(user, self)

    def get_content_metadata_exporter(self, user):
        return CanvasContentMetadataExporter(user, self)

    def get_content_metadata_transmitter(self):
        return CanvasContentMetadataTransmitter(self)


class CanvasLearnerAssessmentDataTransmissionAudit(LearnerDataTransmissionAudit):
    """
    The payload correlated to a courses subsection learner data we send to canvas at a given point in time for an
    enterprise course enrollment.

    .. no_pii:
    """
    canvas_user_email = models.CharField(
        max_length=255,
        blank=False,
        null=False
    )

    grade_point_score = models.FloatField(
        blank=False,
        null=False,
        help_text=_("The amount of points that the learner scored on the subsection.")
    )

    grade_points_possible = models.FloatField(
        blank=False,
        null=False,
        help_text=_("The total amount of points that the learner could score on the subsection.")
    )

    class Meta:
        app_label = 'canvas'

    def __str__(self):
        return (
            '<CanvasLearnerAssessmentDataTransmissionAudit {transmission_id} for enterprise enrollment {enrollment}, '
            'email {canvas_user_email}, and course {course_id}>'.format(
                transmission_id=self.id,
                enrollment=self.enterprise_course_enrollment_id,
                canvas_user_email=self.canvas_user_email,
                course_id=self.course_id
            )
        )

    def __repr__(self):
        """
        Return uniquely identifying string representation.
        """
        return self.__str__()

    def serialize(self, *args, **kwargs):
        """
        Return a JSON-serialized representation.

        Sort the keys so the result is consistent and testable.

        # TODO: When we refactor to use a serialization flow consistent with how course metadata
        # is serialized, remove the serialization here and make the learner data exporter handle the work.
        """
        return json.dumps(self._payload_data(), sort_keys=True)

    def _payload_data(self):
        """
        Convert the audit record's fields into Canvas key/value pairs.
        """
        return {
            'userID': self.canvas_user_email,
            'courseID': self.course_id,
            'grade': self.grade,
            'subsectionID': self.subsection_id,
            'points_possible': self.grade_points_possible,
            'points_earned': self.grade_point_score,
            'subsection_name': self.subsection_name,
        }

    @classmethod
    def audit_type(cls):
        """
        Assessment level audit type labeling
        """
        return "assessment"


class CanvasLearnerDataTransmissionAudit(LearnerDataTransmissionAudit):
    """
    The payload we send to canvas at a given point in time for an enterprise course enrollment.

    .. no_pii:
    """
    canvas_user_email = models.CharField(
        max_length=255,
        blank=False,
        null=False
    )

    canvas_completed_timestamp = models.CharField(
        null=True,
        blank=True,
        max_length=10,
        help_text=_(
            'Represents the canvas representation of a timestamp: yyyy-mm-dd, '
            'which is always 10 characters. Can be left unset for audit transmissions.'
        )
    )

    # XXX non-standard
    grade = models.CharField(max_length=255, null=True, blank=True)

    class Meta:
        app_label = 'canvas'
        constraints = [
            models.UniqueConstraint(
                fields=['enterprise_course_enrollment_id', 'course_id'],
                name='canvas_unique_enrollment_course_id'
            )
        ]
        index_together = ['enterprise_customer_uuid', 'plugin_configuration_id']

    def __str__(self):
        return (
            '<CanvasLearnerDataTransmissionAudit {transmission_id} for enterprise enrollment {enrollment}, '
            'email {canvas_user_email}, and course {course_id}>'.format(
                transmission_id=self.id,
                enrollment=self.enterprise_course_enrollment_id,
                canvas_user_email=self.canvas_user_email,
                course_id=self.course_id
            )
        )

    def __repr__(self):
        """
        Return uniquely identifying string representation.
        """
        return self.__str__()

    def serialize(self, *args, **kwargs):
        """
        Return a JSON-serialized representation.

        Sort the keys so the result is consistent and testable.

        # TODO: When we refactor to use a serialization flow consistent with how course metadata
        # is serialized, remove the serialization here and make the learner data exporter handle the work.
        """
        return json.dumps(self._payload_data(), sort_keys=True)

    def _payload_data(self):
        """
        Convert the audit record's fields into Canvas key/value pairs.
        """
        return {
            'userID': self.canvas_user_email,
            'courseID': self.course_id,
            'courseCompleted': 'true' if self.course_completed else 'false',
            'completedTimestamp': self.canvas_completed_timestamp,
            'grade': self.grade,
        }



# Generated by Django 3.2.12 on 2022-03-10 18:59

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('degreed2', '0007_auto_20220302_2231'),
    ]

    operations = [
        migrations.AddField(
            model_name='degreed2learnerdatatransmissionaudit',
            name='course_completed',
            field=models.BooleanField(default=True, help_text="The learner's course completion status transmitted to Degreed2."),
        ),
    ]



"""
    url mappings for integrated_channels/api/v1/cornerstone/
"""

from rest_framework import routers

from django.urls import path

from .views import CornerstoneConfigurationViewSet, CornerstoneLearnerInformationView

app_name = 'cornerstone'
router = routers.DefaultRouter()
router.register(r'configuration', CornerstoneConfigurationViewSet, basename="configuration")
urlpatterns = [
    path('save-learner-information', CornerstoneLearnerInformationView.as_view(),
         name='save-learner-information'
         ),
]

urlpatterns += router.urls



"""
API clients to communicate with Braze services.
"""
import logging

from braze.client import BrazeClient

from django.conf import settings

logger = logging.getLogger(__name__)

ENTERPRISE_BRAZE_ALIAS_LABEL = 'Enterprise'  # Do Not change this, this is consistent with other uses across edX repos.


class BrazeAPIClient(BrazeClient):
    """
    API client for calls to Braze.
    """
    def __init__(self):
        braze_api_key = getattr(settings, 'EDX_BRAZE_API_KEY', None)
        braze_api_url = getattr(settings, 'EDX_BRAZE_API_SERVER', None)
        required_settings = ['EDX_BRAZE_API_KEY', 'EDX_BRAZE_API_SERVER']
        for setting in required_settings:
            if not getattr(settings, setting, None):
                msg = f'Missing {setting} in settings required for Braze API Client.'
                logger.error(msg)
                raise ValueError(msg)

        super().__init__(
            api_key=braze_api_key,
            api_url=braze_api_url,
            app_id='',
        )

    def generate_mailto_link(self, emails):
        """
        Generate a mailto link for the given emails.
        """
        if emails:
            return f'mailto:{",".join(emails)}'

        return None

    def create_recipient(
        self,
        user_email,
        lms_user_id,
        trigger_properties=None,
    ):
        """
        Create a recipient object using the given user_email and lms_user_id.
        Identifies the given email address with any existing Braze alias records
        via the provided ``lms_user_id``.
        """

        user_alias = {
            'alias_label': ENTERPRISE_BRAZE_ALIAS_LABEL,
            'alias_name': user_email,
        }

        # Identify the user alias in case it already exists. This is necessary so
        # we don't accidentally create a duplicate Braze profile.
        self.identify_users([{
            'external_id': lms_user_id,
            'user_alias': user_alias
        }])

        attributes = {
            "user_alias": user_alias,
            "email": user_email,
            "is_enterprise_learner": True,
            "_update_existing_only": False,
        }

        return {
            'external_user_id': lms_user_id,
            'attributes': attributes,
            # If a profile does not already exist, Braze will create a new profile before sending a message.
            'send_to_existing_only': False,
            'trigger_properties': trigger_properties or {},
        }

    def create_recipient_no_external_id(self, user_email):
        """
        Create a Braze recipient dict identified only by an alias based on their email.
        """
        return {
            'attributes': {
                'email': user_email,
                'is_enterprise_learner': True,
            },
            'user_alias': {
                'alias_label': ENTERPRISE_BRAZE_ALIAS_LABEL,
                'alias_name': user_email,
            },
        }



"""
These settings are here to use during tests, because django requires them.

In a real-world use case, apps in this project are installed into other
Django applications, so these settings will not be used.
"""

import os
from os.path import abspath, dirname, join

from celery import Celery

from enterprise.constants import (
    ENTERPRISE_ADMIN_ROLE,
    ENTERPRISE_CATALOG_ADMIN_ROLE,
    ENTERPRISE_DASHBOARD_ADMIN_ROLE,
    ENTERPRISE_ENROLLMENT_API_ADMIN_ROLE,
    ENTERPRISE_FULFILLMENT_OPERATOR_ROLE,
    ENTERPRISE_OPERATOR_ROLE,
    ENTERPRISE_SSO_ORCHESTRATOR_OPERATOR_ROLE,
)


def here(*args):
    """
    Return the absolute path to a directory from this file.
    """
    return join(abspath(dirname(__file__)), *args)


def root(*args):
    """
    Return the absolute path to some file from the project's root.
    """
    return abspath(join(abspath(here('../..')), *args))


DATABASES = {
    'default': {
        'ENGINE': os.environ.get('DB_ENGINE', 'django.db.backends.sqlite3'),
        'NAME': os.environ.get('DB_NAME', 'default.db'),
        'USER': os.environ.get('DB_USER', ''),
        'PASSWORD': os.environ.get('DB_PASSWORD', ''),
        'HOST': os.environ.get('DB_HOST', ''),
        'PORT': os.environ.get('DB_PORT', ''),
    },
}


INSTALLED_APPS = (
    "django.contrib.auth",
    "django.contrib.contenttypes",
    "django.contrib.sites",
    "django.contrib.sessions",
    "django.contrib.admin",  # only used in DEBUG mode
    "django.contrib.messages",
    "django.contrib.staticfiles",
    "waffle",

    "enterprise",
    "consent",
    "integrated_channels.integrated_channel",
    "integrated_channels.cornerstone",
    "integrated_channels.degreed",
    "integrated_channels.degreed2",
    "integrated_channels.canvas",
    "integrated_channels.blackboard",
    "integrated_channels.moodle",
    "integrated_channels.sap_success_factors",
    "integrated_channels.xapi",
    "edx_rbac",
    "rules.apps.AutodiscoverRulesConfig",
    "oauth2_provider",
)

MIDDLEWARE_CLASSES = [
    "django.contrib.sessions.middleware.SessionMiddleware",
    "django.contrib.auth.middleware.AuthenticationMiddleware",
    "django.contrib.messages.middleware.MessageMiddleware",
    "crum.CurrentRequestUserMiddleware",
    "waffle.middleware.WaffleMiddleware",
]

MIDDLEWARE = MIDDLEWARE_CLASSES  # Django 1.10 compatibility - the setting was renamed

AUTHENTICATION_BACKENDS = (
    "rules.permissions.ObjectPermissionBackend",
    "django.contrib.auth.backends.ModelBackend",
)

SITE_NAME = 'localhost:18000/'

SESSION_ENGINE = "django.contrib.sessions.backends.file"

LOCALE_PATHS = [
    root("enterprise", "conf", "locale"),
]

MAKO_TEMPLATES = {
    "main": []
}

TEMPLATES = [
    {
        "BACKEND": "django.template.backends.django.DjangoTemplates",
        "APP_DIRS": True,
        "OPTIONS": {
            "context_processors": {
                "django.contrib.auth.context_processors.auth",  # this is required for admin
                "django.contrib.messages.context_processors.messages",
            }
        }
    },
]

REPO_ROOT = root('enterprise')

STATIC_ROOT = root('enterprise/assets')

STATIC_URL = '/enterprise/static/'

PLATFORM_NAME = "Test platform"
PLATFORM_DESCRIPTION = "Test description"

ROOT_URLCONF = "enterprise.urls"

SECRET_KEY = "insecure-secret-key"

# Default Site id
SITE_ID = 1

EDX_API_KEY = "PUT_YOUR_API_KEY_HERE"

COURSE_CATALOG_URL_ROOT = "http://localhost:18381/"
COURSE_CATALOG_API_URL = "http://localhost:18381/api/v1/"

LMS_ROOT_URL = "http://lms.example.com"
LMS_INTERNAL_ROOT_URL = "http://localhost:8000"
LMS_ENROLLMENT_API_PATH = "/api/enrollment/v1/"
ECOMMERCE_PUBLIC_URL_ROOT = "http://localhost:18130"
ENTERPRISE_CATALOG_INTERNAL_ROOT_URL = "http://localhost:18160"

ENTERPRISE_ENROLLMENT_API_URL = LMS_INTERNAL_ROOT_URL + LMS_ENROLLMENT_API_PATH

ENTERPRISE_LEARNER_PORTAL_BASE_URL = 'http://localhost:8734'

ENTERPRISE_PUBLIC_ENROLLMENT_API_URL = ENTERPRISE_ENROLLMENT_API_URL

ENTERPRISE_API_CACHE_TIMEOUT = 60

ENTERPRISE_SUPPORT_URL = "http://foo"

ENTERPRISE_TAGLINE = "High-quality online learning opportunities from the world's best universities"

OAUTH_ID_TOKEN_EXPIRATION = 60 * 60  # in seconds

EMAIL_BACKEND = 'django.core.mail.backends.locmem.EmailBackend'

DEFAULT_FROM_EMAIL = 'course_staff@example.com'

USER_THROTTLE_RATE = '190/minute'
SERVICE_USER_THROTTLE_RATE = '200/minute'
SERVICE_USER_HIGH_THROTTLE_RATE = '200/minute'
REST_FRAMEWORK = {
    'DEFAULT_PAGINATION_CLASS': 'rest_framework.pagination.PageNumberPagination',
    'PAGE_SIZE': 10,
    'URL_FORMAT_OVERRIDE': None,
    'DEFAULT_THROTTLE_CLASSES': (
        'rest_framework.throttling.UserRateThrottle',
    ),
    'DEFAULT_THROTTLE_RATES': {
        'user': USER_THROTTLE_RATE,
        'service_user': SERVICE_USER_THROTTLE_RATE,
        'high_service_user': SERVICE_USER_HIGH_THROTTLE_RATE,
    },
    'DATETIME_FORMAT': '%Y-%m-%dT%H:%M:%SZ',
}

# URL for the server that django client listens to by default.
TEST_SERVER = "http://testserver"
ALLOWED_HOSTS = ["testserver.enterprise"]
MEDIA_URL = "/"

# Defines the usernames of service users who should be throttled
# at a higher rate than normal users.
ENTERPRISE_ALL_SERVICE_USERNAMES = [
    'ecommerce_worker',
    'enterprise_worker',
    'license_manager_worker',
    'enterprise_catalog_worker',
    'enterprise_subsidy_worker',
]

ECOMMERCE_SERVICE_WORKER_USERNAME = 'ecommerce_worker'
ENTERPRISE_SERVICE_WORKER_USERNAME = 'enterprise_worker'

ENTERPRISE_CUSTOMER_LOGO_IMAGE_SIZE = 512   # Enterprise logo image size limit in KB's

ENTERPRISE_COURSE_ENROLLMENT_AUDIT_MODES = ['audit', 'honor']

# These are standard regexes for pulling out info like course_ids, usage_ids, etc.
COURSE_KEY_PATTERN = r'(?P<course_key_string>[^/+]+(/|\+)[^/+]+(/|\+)[^/?]+)'
COURSE_ID_PATTERN = COURSE_KEY_PATTERN.replace('course_key_string', 'course_id')

USE_TZ = True

TIME_ZONE = 'UTC'

MKTG_URLS = {}

ENTERPRISE_CUSTOMER_CATALOG_DEFAULT_CONTENT_FILTER = {
    'content_type': 'course',
    'partner': 'edx',
    'level_type': [
        'Introductory',
        'Intermediate',
        'Advanced'
    ],
    'availability': [
        'Current',
        'Starting Soon',
        'Upcoming'
    ],
    'status': 'published'
}

SNOWFLAKE_SERVICE_USER = 'TEST@EDX.ORG'
SNOWFLAKE_SERVICE_USER_PASSWORD = 'secret'

# For testing edx-rbac rules. This is not the actual value of the setting in prod.
SYSTEM_TO_FEATURE_ROLE_MAPPING = {
    ENTERPRISE_ADMIN_ROLE: [
        ENTERPRISE_DASHBOARD_ADMIN_ROLE,
        ENTERPRISE_CATALOG_ADMIN_ROLE,
        ENTERPRISE_ENROLLMENT_API_ADMIN_ROLE
    ],
    ENTERPRISE_OPERATOR_ROLE: [
        ENTERPRISE_DASHBOARD_ADMIN_ROLE,
        ENTERPRISE_CATALOG_ADMIN_ROLE,
        ENTERPRISE_ENROLLMENT_API_ADMIN_ROLE,
        ENTERPRISE_FULFILLMENT_OPERATOR_ROLE,
        ENTERPRISE_SSO_ORCHESTRATOR_OPERATOR_ROLE,
    ],
}

LOGGING = {
    'version': 1,
    'disable_existing_loggers': False,
    'handlers': {
        'console': {
            'level': 'DEBUG',
            'class': 'logging.StreamHandler',
        },
    },
    'loggers': {
        'rules': {
            'handlers': ['console'],
            'level': 'DEBUG',
            'propagate': True,
        },
    },
}

################################### TRACKING ###################################

LMS_SEGMENT_KEY = 'SOME_KEY'
EVENT_TRACKING_ENABLED = True
EVENT_TRACKING_BACKENDS = {
    'segmentio': {
        'ENGINE': 'eventtracking.backends.routing.RoutingBackend',
        'OPTIONS': {
            'backends': {
                'segment': {'ENGINE': 'eventtracking.backends.segment.SegmentBackend'}
            },
            'processors': [
                {
                    'ENGINE': 'eventtracking.processors.whitelist.NameWhitelistProcessor',
                    'OPTIONS': {
                        'whitelist': []
                    }
                },
            ]
        }
    }
}
EVENT_TRACKING_PROCESSORS = []

#################################### CELERY ####################################

app = Celery('enterprise')
app.conf.task_protocol = 1
app.config_from_object('django.conf:settings')

CELERY_ALWAYS_EAGER = True

CLEAR_REQUEST_CACHE_ON_TASK_COMPLETION = False

##### END CELERY #####

JWT_AUTH = {
    'JWT_AUDIENCE': 'test-aud',
    'JWT_DECODE_HANDLER': 'edx_rest_framework_extensions.auth.jwt.decoder.jwt_decode_handler',
    'JWT_ISSUER': 'test-iss',
    'JWT_LEEWAY': 1,
    'JWT_SECRET_KEY': 'test-key',
    'JWT_SUPPORTED_VERSION': '1.0.0',
    'JWT_VERIFY_AUDIENCE': False,
    'JWT_VERIFY_EXPIRATION': True,

    # JWT_ISSUERS enables token decoding for multiple issuers (Note: This is not a native DRF-JWT field)
    # We use it to allow different values for the 'ISSUER' field, but keep the same SECRET_KEY and
    # AUDIENCE values across all issuers.
    'JWT_ISSUERS': [
        {
            'ISSUER': 'test-issuer-1',
            'SECRET_KEY': 'test-secret-key',
            'AUDIENCE': 'test-audience',
        },
        {
            'ISSUER': 'test-issuer-2',
            'SECRET_KEY': 'test-secret-key',
            'AUDIENCE': 'test-audience',
        }
    ],
}


INTEGRATED_CHANNELS_API_CHUNK_TRANSMISSION_LIMIT = {
    'SAP': 1,
}

LANGUAGE_COOKIE = 'openedx-language-preference'

ENTERPRISE_BACKEND_SERVICE_EDX_OAUTH2_PROVIDER_URL = f'{LMS_INTERNAL_ROOT_URL}/oauth2'
ENTERPRISE_BACKEND_SERVICE_EDX_OAUTH2_KEY = 'test_backend_oauth2_key'
ENTERPRISE_BACKEND_SERVICE_EDX_OAUTH2_SECRET = 'test_backend_oauth2_secret'

ECOMMERCE_API_URL = 'https://ecommerce.example.com/api/v2/'

LOGIN_REDIRECT_WHITELIST = [
    'success.url',
    'failure.url',
    'google.com',
    'facebook.com'
]

ENTERPRISE_PLOTLY_SECRET = "I am a secret"
ENTERPRISE_MANUAL_REPORTING_CUSTOMER_UUIDS = ['12aacfee8ffa4cb3bed1059565a57f06',]
EXEC_ED_LANDING_PAGE = 'https://www.edx-external.com/account'


# disable indexing on history_date. Otherwise it will add new alter migrations.
SIMPLE_HISTORY_DATE_INDEX = False

OPENAI_API_KEY = ''

LEARNER_ENGAGEMENT_PROMPT_FOR_ACTIVE_CONTRACT = 'Create learner engagement report for active contracts.'
LEARNER_ENGAGEMENT_PROMPT_FOR_NON_ACTIVE_CONTRACT = 'Create learner engagement report for non active contracts.'
LEARNER_PROGRESS_PROMPT_FOR_ACTIVE_CONTRACT = 'Create learner progress report for active contracts.'
LEARNER_PROGRESS_PROMPT_FOR_NON_ACTIVE_CONTRACT = 'Create learner progress report for non active contracts.'

ENTERPRISE_SSO_ORCHESTRATOR_WORKER_USERNAME = 'username'
ENTERPRISE_SSO_ORCHESTRATOR_WORKER_PASSWORD = 'password'
ENTERPRISE_SSO_ORCHESTRATOR_BASE_URL = 'https://foobar.com'
ENTERPRISE_SSO_ORCHESTRATOR_CONFIGURE_PATH = 'configure'
ENTERPRISE_SSO_ORCHESTRATOR_CONFIGURE_EDX_OAUTH_PATH = 'configure-edx-oauth'

EDX_BRAZE_API_KEY='test-api-key'
EDX_BRAZE_API_SERVER='test-api-server'
BRAZE_GROUPS_INVITATION_EMAIL_CAMPAIGN_ID = 'test-invitation-campaign-id'
BRAZE_GROUPS_REMOVAL_EMAIL_CAMPAIGN_ID = 'test-removal-campaign-id'



"""
Mark for re-send all CSOD content transmission with a remote_deleted_at but no api_response_status_code
"""
import logging

from django.apps import apps
from django.contrib import auth
from django.core.management.base import BaseCommand
from django.db.models import Q

from integrated_channels.integrated_channel.management.commands import IntegratedChannelCommandMixin
from integrated_channels.utils import batch_by_pk, generate_formatted_log

User = auth.get_user_model()

LOGGER = logging.getLogger(__name__)


class Command(IntegratedChannelCommandMixin, BaseCommand):
    """
    Mark for re-send all CSOD content transmission with a remote_deleted_at but no api_response_status_code

    ./manage.py lms reset_csod_remote_deleted_at
    """

    def handle(self, *args, **options):
        """
        Mark for re-send all CSOD content transmission with a remote_deleted_at
        """

        ContentMetadataItemTransmission = apps.get_model(
            'integrated_channel',
            'ContentMetadataItemTransmission'
        )

        csod_deleted_at = Q(
            integrated_channel_code='CSOD',
            remote_deleted_at__isnull=False
        )

        for items_batch in batch_by_pk(ContentMetadataItemTransmission, extra_filter=csod_deleted_at):
            for item in items_batch:
                try:
                    item.remote_deleted_at = None
                    item.save()
                    LOGGER.info(generate_formatted_log(
                        item.integrated_channel_code,
                        item.enterprise_customer.uuid,
                        None,
                        item.content_id,
                        f'integrated_channel_content_transmission_id={item.id}, '
                        'setting remote_deleted_at to None'
                    ))
                except Exception:  # pylint: disable=broad-except
                    LOGGER.exception(generate_formatted_log(
                        item.integrated_channel_code,
                        item.enterprise_customer.uuid,
                        None,
                        item.content_id,
                        f'integrated_channel_content_transmission_id={item.id}, '
                        'error setting remote_deleted_at to None'
                    ))



"""
Content metadata exporter for Cornerstone.
"""

import datetime
from logging import getLogger

import pytz

from django.apps import apps
from django.conf import settings

from enterprise.utils import get_closest_course_run, get_language_code
from integrated_channels.cornerstone.utils import convert_invalid_course_id
from integrated_channels.integrated_channel.constants import ISO_8601_DATE_FORMAT
from integrated_channels.integrated_channel.exporters.content_metadata import ContentMetadataExporter
from integrated_channels.utils import (
    get_duration_from_estimated_hours,
    get_image_url,
    get_subjects_from_content_metadata,
)

LOGGER = getLogger(__name__)


class CornerstoneContentMetadataExporter(ContentMetadataExporter):
    """
    Cornerstone implementation of ContentMetadataExporter.
    """
    LONG_STRING_LIMIT = 10000
    DEFAULT_SUBJECT = "Industry Specific"
    DEFAULT_LANGUAGE = "English"
    DEFAULT_PARTNER = {
        "Name": "edX: edX Inc"
    }
    DATA_TRANSFORM_MAPPING = {
        'ID': 'key',
        'Title': 'title',
        'Description': 'description',
        'Thumbnail': 'image',
        'URL': 'enrollment_url',
        'IsActive': 'is_active',
        'LastModifiedUTC': 'modified',
        'Duration': 'estimated_hours',
        'Partners': 'organizations',
        'Languages': 'languages',
        'Subjects': 'subjects',
    }
    SKIP_KEY_IF_NONE = True
    MAX_PAYLOAD_COUNT = getattr(settings, "ENTERPRISE_CORNERSTONE_MAX_CONTENT_PAYLOAD_COUNT", 1000)

    def export_for_web_polling(self, max_payload_count=MAX_PAYLOAD_COUNT):  # pylint: disable=unused-argument
        """
        Return the exported and transformed content metadata as a dictionary for CSDO web pull.
        """
        ContentMetadataItemTransmission = apps.get_model(
            'integrated_channel',
            'ContentMetadataItemTransmission'
        )
        create_payload = ContentMetadataItemTransmission.objects.filter(
            marked_for='create',
            enterprise_customer=self.enterprise_configuration.enterprise_customer,
            plugin_configuration_id=self.enterprise_configuration.id,
        )
        update_payload = ContentMetadataItemTransmission.objects.filter(
            marked_for='update',
            enterprise_customer=self.enterprise_configuration.enterprise_customer,
            plugin_configuration_id=self.enterprise_configuration.id,
        )
        delete_payload = ContentMetadataItemTransmission.objects.filter(
            marked_for='delete',
            enterprise_customer=self.enterprise_configuration.enterprise_customer,
            plugin_configuration_id=self.enterprise_configuration.id,
        )
        created = {record.content_id: record for record in create_payload}
        updated = {record.content_id: record for record in update_payload}
        deleted = {record.content_id: record for record in delete_payload}
        return created, updated, deleted

    def transform_courserun_key(self, content_metadata_item):
        """
        Return the transformed version of the course run key by converting into a string of valid chars by encoding the
        key with base 64. Because valid course run keys have already been transmitted as courses, and course keys are
        used to uniquely identify edx courses, we only want to encode the invalid ones as they would have never been
        created.
        """
        return convert_invalid_course_id(content_metadata_item.get('key'))

    def transform_course_key(self, content_metadata_item):
        """
        Return the transformed version of the course key by converting into a string of valid chars by encoding the key
        with base 64
        """
        return convert_invalid_course_id(content_metadata_item.get('key'))

    def transform_organizations(self, content_metadata_item):
        """
        Return the transformed version of the course organizations by converting each organization into cornerstone
        course partner object. or default Partner if no partner found
        """
        partners = []
        for org in content_metadata_item.get('organizations') or []:
            org_name = org[:500] if org else ''
            partners.append({"Name": org_name})
        return partners or [self.DEFAULT_PARTNER]

    def transform_is_active(self, content_metadata_item):
        """
        Return the transformed version of the course is_active status by traversing course runs and setting IsActive to
        True if any of the course runs have availability value set to `Current`, `Starting Soon` or `Upcoming`.
        """
        is_active = False
        for course_run in content_metadata_item.get('course_runs', []):
            if course_run.get('availability') in ['Current', 'Starting Soon', 'Upcoming']:
                is_active = True
                break
        return is_active

    def transform_modified(self, content_metadata_item):
        """
        Return the modified datetime of closest course run`.
        """
        modified_datetime = datetime.datetime.now(pytz.UTC).strftime(ISO_8601_DATE_FORMAT)
        course_runs = content_metadata_item.get('course_runs')
        if course_runs:
            closest_course_run = get_closest_course_run(course_runs)
            modified_datetime = closest_course_run.get('modified', modified_datetime)

        return str(modified_datetime)

    def transform_estimated_hours(self, content_metadata_item):
        """
        Return the duration of course in hh:mm:ss format.
        """
        duration = None
        course_runs = content_metadata_item.get('course_runs')
        if course_runs:
            closest_course_run = get_closest_course_run(course_runs)
            estimated_hours = closest_course_run.get('estimated_hours')
            duration = get_duration_from_estimated_hours(estimated_hours)

        return duration

    def transform_image(self, content_metadata_item):
        """
        Return the image URI of the content item.
        """
        return get_image_url(content_metadata_item)

    def transform_languages(self, content_metadata_item):
        """
        Return the languages supported by course or `English` as default if no languages found.
        """
        CornerstoneGlobalConfiguration = apps.get_model(
            'cornerstone',
            'CornerstoneGlobalConfiguration'
        )
        languages_json = CornerstoneGlobalConfiguration.current().languages or {'Languages': []}

        languages = content_metadata_item.get('languages') or [self.DEFAULT_LANGUAGE]
        course_languages = [get_language_code(language) for language in languages]
        languages = set(languages_json['Languages']).intersection(set(course_languages))
        return list(languages) if languages else ['en-US']

    def transform_description(self, content_metadata_item):
        """
        Return the transformed version of the course description.

        We choose one value out of the course's full description, short description, and title depending on availability
        and length limits.
        """
        full_description = content_metadata_item.get('full_description') or ''
        if 0 < len(full_description) <= self.LONG_STRING_LIMIT:
            return full_description
        return content_metadata_item.get('short_description') or content_metadata_item.get('title') or ''

    def transform_subjects(self, content_metadata_item):
        """
        Return the transformed version of the course subject list or default value if no subject found.
        """
        if self.enterprise_configuration.disable_subject_metadata_transmission:
            return None
        subjects = []
        course_subjects = get_subjects_from_content_metadata(content_metadata_item)
        CornerstoneGlobalConfiguration = apps.get_model(
            'cornerstone',
            'CornerstoneGlobalConfiguration'
        )
        subjects_mapping_dict = CornerstoneGlobalConfiguration.current().subject_mapping or {}
        for subject in course_subjects:
            for cornerstone_subject, edx_subjects in subjects_mapping_dict.items():
                if subject.lower() in [edx_subject.lower() for edx_subject in edx_subjects]:
                    subjects.append(cornerstone_subject)
        return list(set(subjects)) or [self.DEFAULT_SUBJECT]



# Generated by Django 2.2.15 on 2020-08-05 18:36

import simple_history.models

import django.db.models.deletion
import django.utils.timezone
from django.conf import settings
from django.db import migrations, models

import model_utils.fields


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ('enterprise', '0107_remove_branding_config_banner_fields'),
    ]

    operations = [
        migrations.CreateModel(
            name='HistoricalCanvasEnterpriseCustomerConfiguration',
            fields=[
                ('id', models.IntegerField(auto_created=True, blank=True, db_index=True, verbose_name='ID')),
                ('created', model_utils.fields.AutoCreatedField(default=django.utils.timezone.now, editable=False, verbose_name='created')),
                ('modified', model_utils.fields.AutoLastModifiedField(default=django.utils.timezone.now, editable=False, verbose_name='modified')),
                ('active', models.BooleanField(help_text='Is this configuration active?')),
                ('transmission_chunk_size', models.IntegerField(default=500, help_text='The maximum number of data items to transmit to the integrated channel with each request.')),
                ('channel_worker_username', models.CharField(blank=True, help_text='Enterprise channel worker username to get JWT tokens for authenticating LMS APIs.', max_length=255, null=True)),
                ('catalogs_to_transmit', models.TextField(blank=True, help_text='A comma-separated list of catalog UUIDs to transmit.', null=True)),
                ('client_id', models.CharField(help_text='The API Client ID provided to edX by the enterprise customer to be used to make API calls to Canvas on behalf of the customer.', max_length=255, null=True, verbose_name='API Client ID')),
                ('client_secret', models.CharField(help_text='The API Client Secret provided to edX by the enterprise customer to be used to make  API calls to Canvas on behalf of the customer.', max_length=255, null=True, verbose_name='API Client Secret')),
                ('canvas_account_id', models.IntegerField(help_text='Account number to use during api calls. Called account_id in canvas.  Required to create courses etc.', null=True, verbose_name='Canvas Account Number')),
                ('canvas_base_url', models.CharField(help_text='The base URL used for API requests to Canvas, i.e. https://instructure.com.', max_length=255, null=True, verbose_name='Canvas Base URL')),
                ('history_id', models.AutoField(primary_key=True, serialize=False)),
                ('history_date', models.DateTimeField()),
                ('history_change_reason', models.CharField(max_length=100, null=True)),
                ('history_type', models.CharField(choices=[('+', 'Created'), ('~', 'Changed'), ('-', 'Deleted')], max_length=1)),
                ('enterprise_customer', models.ForeignKey(blank=True, db_constraint=False, help_text='Enterprise Customer associated with the configuration.', null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='+', to='enterprise.EnterpriseCustomer')),
                ('history_user', models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='+', to=settings.AUTH_USER_MODEL)),
            ],
            options={
                'verbose_name': 'historical canvas enterprise customer configuration',
                'get_latest_by': 'history_date',
                'ordering': ('-history_date', '-history_id'),
            },
            bases=(simple_history.models.HistoricalChanges, models.Model),
        ),
        migrations.CreateModel(
            name='CanvasGlobalConfiguration',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('change_date', models.DateTimeField(auto_now_add=True, verbose_name='Change date')),
                ('enabled', models.BooleanField(default=False, verbose_name='Enabled')),
                ('course_api_path', models.CharField(help_text='The API path for making course metadata POST/DELETE requests to Canvas.', max_length=255, verbose_name='Course Metadata API Path')),
                ('changed_by', models.ForeignKey(editable=False, null=True, on_delete=django.db.models.deletion.PROTECT, to=settings.AUTH_USER_MODEL, verbose_name='Changed by')),
            ],
        ),
        migrations.CreateModel(
            name='CanvasEnterpriseCustomerConfiguration',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created', model_utils.fields.AutoCreatedField(default=django.utils.timezone.now, editable=False, verbose_name='created')),
                ('modified', model_utils.fields.AutoLastModifiedField(default=django.utils.timezone.now, editable=False, verbose_name='modified')),
                ('active', models.BooleanField(help_text='Is this configuration active?')),
                ('transmission_chunk_size', models.IntegerField(default=500, help_text='The maximum number of data items to transmit to the integrated channel with each request.')),
                ('channel_worker_username', models.CharField(blank=True, help_text='Enterprise channel worker username to get JWT tokens for authenticating LMS APIs.', max_length=255, null=True)),
                ('catalogs_to_transmit', models.TextField(blank=True, help_text='A comma-separated list of catalog UUIDs to transmit.', null=True)),
                ('client_id', models.CharField(help_text='The API Client ID provided to edX by the enterprise customer to be used to make API calls to Canvas on behalf of the customer.', max_length=255, null=True, verbose_name='API Client ID')),
                ('client_secret', models.CharField(help_text='The API Client Secret provided to edX by the enterprise customer to be used to make  API calls to Canvas on behalf of the customer.', max_length=255, null=True, verbose_name='API Client Secret')),
                ('canvas_account_id', models.IntegerField(help_text='Account number to use during api calls. Called account_id in canvas.  Required to create courses etc.', null=True, verbose_name='Canvas Account Number')),
                ('canvas_base_url', models.CharField(help_text='The base URL used for API requests to Canvas, i.e. https://instructure.com.', max_length=255, null=True, verbose_name='Canvas Base URL')),
                ('enterprise_customer', models.OneToOneField(help_text='Enterprise Customer associated with the configuration.', on_delete=django.db.models.deletion.CASCADE, to='enterprise.EnterpriseCustomer')),
            ],
        ),
    ]
